/* 
 * This file is part of fuelpad, the fuel diary
 *
 * Copyright (c) 2007, 2008 Julius Luukko <julle.luukko@quicknet.inet.fi>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/**
 *
 * \file
 * \brief  Main of fuelpad
 * \author Julius Luukko
 *
 */

#include <config.h>

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>

#include <locale.h>

#include <sqlite3.h>

#include "fuelpad.h"
#include "ui.h"

int debug_level;

/* local function prototypes */
void db_connect(void);

unsigned int currentdriver=1;
unsigned int currentcar=1;

sqlite3 *db;
#if PLAINGTK == 1
char default_db[] = "fuelpad.db";
#else
char default_db[] = "/home/user/MyDocs/fuelpad.db";
#endif
char *result_db;
sqlite3_stmt *ppStmtSaveRecord;
sqlite3_stmt *ppStmtDriver;
sqlite3_stmt *ppStmtCar;
sqlite3_stmt *ppStmtOneDriver;
sqlite3_stmt *ppStmtCurCar;
sqlite3_stmt *ppStmtOneCar;
sqlite3_stmt *ppStmtAddDriver;
sqlite3_stmt *ppStmtUpdateDriver;
sqlite3_stmt *ppStmtAddCar;
sqlite3_stmt *ppStmtUpdateCar;
sqlite3_stmt *ppStmtOneRecord;
sqlite3_stmt *ppStmtRecords;
sqlite3_stmt *ppStmtExport;
sqlite3_stmt *ppStmtExportCar;
sqlite3_stmt *ppStmtAddRecord;
sqlite3_stmt *ppStmtDeleteRecord;
sqlite3_stmt *ppStmtUpdateRecord;
sqlite3_stmt *ppStmtGetYears;
sqlite3_stmt *ppStmtGetReport;
sqlite3_stmt *ppStmtAddAlarmtype;
sqlite3_stmt *ppStmtGetAlarmtype;
sqlite3_stmt *ppStmtGetOneAlarmtype;
sqlite3_stmt *ppStmtUpdateAlarmtype;
sqlite3_stmt *ppStmtAddEvent;
sqlite3_stmt *ppStmtGetEvents;
sqlite3_stmt *ppStmtGetOneEvent;
sqlite3_stmt *ppStmtDeleteEvent;
sqlite3_stmt *ppStmtDeleteEventwithRecordid;
sqlite3_stmt *ppStmtUpdateEvent;

/**
 * \fn double atof_l(const char *nptr, const char *locale)
 * \brief Opens the connection to sqlite3 database
 * \param *nptr string to be converted
 * \param *locale locale used in conversion
 * \return String as numeric value
 *
 * Locale independent atof. Changes the locale to *locale, makes the
 * conversion and changes the locale back to original.
 *
 */
double atof_l(const char *nptr, const char *locale)
{
  char *currentlocale;
  char *psave;
  size_t len;
  double ret;

  currentlocale=setlocale(LC_NUMERIC, NULL);
  len=strlen(currentlocale);
  psave=(char *)malloc((len+1)*sizeof(char));
  strncpy(psave, currentlocale, len+1);

  setlocale(LC_NUMERIC, locale);
  ret=atof(nptr);

  setlocale(LC_NUMERIC, psave);
  free(psave);

  return ret;
}

/**
 * \fn void db_setfilename(void)
 * \brief Opens the connection to sqlite3 database
 *
 * Opens the connection to sqlite3 database. If a database does not
 * exist, creates a new one. Leaves the database open.
 *
 */
void db_setfilename(char *fn)
{
  size_t len;

  if (fn==NULL)
    fn=default_db;

  len=strlen(fn);
  result_db=(char *)realloc((void *)result_db, len+1);
  strncpy(result_db, fn, len+1);
}

char *db_getfilename(void)
{
  return result_db;
}

/**
 * \fn void db_connect(void)
 * \brief Opens the connection to sqlite3 database
 *
 * Opens the connection to sqlite3 database. If a database does not
 * exist, creates a new one. Leaves the database open.
 *
 */
void db_connect(void)
{
  char *zErrMsg = NULL;
  char **sqlresult;
  int sqlrows;
  int sqlcolumns;
  int rc;
  char *create_sql;
  int maxid;

  if (db) {
    sqlite3_close(db);
    db=NULL;
  }

  if (SQLITE_OK != (sqlite3_open(result_db,&db))) {
    PDEBUG("Problem opening result database.");
    sqlite3_close(db);
    db=NULL;
  }

  if (db) {
    /* Check if database does not exist */
    rc = sqlite3_get_table(db, "SELECT nickname FROM driver LIMIT 1",&sqlresult, &sqlrows, &sqlcolumns, NULL);
    sqlite3_free_table(sqlresult);
    if (rc != SQLITE_OK ) { /* create database if not found */
      create_sql = sqlite3_mprintf(
				   "CREATE TABLE driver ("
				   "id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,"
				   "fullname TEXT,"
				   "nickname TEXT);"
				   "CREATE TABLE car ("
				   "id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,"
				   "mark TEXT,"
				   "model TEXT,"
				   "year NUMBER,"
				   "register TEXT,"
				   "notes TEXT);"
				   "CREATE TABLE record("
				   "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   "carid INTEGER,"
				   "driverid INTEGER,"
				   "day TIMESTAMP,"
				   "km REAL,"
				   "trip REAL,"
				   "fill REAL,"
				   "consum REAL,"
				   "price REAL,"
				   "priceperlitre REAL,"
				   "service REAL,"
				   "oil REAL,"
				   "tires REAL,"
				   "insurance REAL,"
				   "other REAL,"
				   "notes TEXT);"
				   "CREATE TABLE alarmtype ("
				   "id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,"
				   "carid INTEGER,"
				   "shortdesc TEXT,"
				   "distance NUMBER,"
				   "interval INTEGER,"
				   "longdesc TEXT);"
				   "CREATE TABLE alarmevent ("
				   "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   "alarmid INTEGER,"
				   "carid INTEGER,"
				   "driverid INTEGER,"
				   "recordid INTEGER,"
				   "day TIMESTAMP,"
				   "km REAL);"
				   "INSERT INTO driver(fullname,nickname)"
				   "VALUES('Default Driver','Default');"
				   "INSERT INTO car(mark,model,year,register,notes)"
				   "VALUES('Default','Model',2007,'ABC-123','');"
				   );

      if (SQLITE_OK != sqlite3_exec(db, create_sql, NULL, NULL, &zErrMsg) &&
	  SQLITE_OK != sqlite3_get_table(db, "SELECT nickname FROM driver LIMIT 1",&sqlresult, &sqlrows, &sqlcolumns, NULL)) {
	PDEBUG("Failed to open or create result database\n");
	PDEBUG("%s\n",sqlite3_errmsg(db));
	sqlite3_close(db);
	db = NULL;
      }
      sqlite3_free(create_sql);
      sqlite3_free_table(sqlresult);
    }
    else { /* Check if database is from version 0.51 or below and in that case
	      create the alarm system tables */
      rc = sqlite3_get_table(db, "SELECT id FROM alarmtype LIMIT 1",&sqlresult, &sqlrows, &sqlcolumns, NULL);
      sqlite3_free_table(sqlresult);
      if (rc != SQLITE_OK ) { /* create new tables if not found */
	PDEBUG("Creating new alarm tables\n");
	create_sql = sqlite3_mprintf(
				   "CREATE TABLE alarmtype ("
				   "id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,"
				   "carid INTEGER,"
				   "shortdesc TEXT,"
				   "distance NUMBER,"
				   "interval INTEGER,"
				   "longdesc TEXT);"
				   "CREATE TABLE alarmevent ("
				   "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   "alarmid INTEGER,"
				   "carid INTEGER,"
				   "driverid INTEGER,"
				   "recordid INTEGER,"
				   "day TIMESTAMP,"
				   "km REAL);"
				   );
	if (SQLITE_OK != sqlite3_exec(db, create_sql, NULL, NULL, &zErrMsg) &&
	    SQLITE_OK != sqlite3_get_table(db, "id FROM alarmtype LIMIT 1",&sqlresult, &sqlrows, &sqlcolumns, NULL)) {
	  PDEBUG("Failed to open or create database\n");
	  PDEBUG("%s\n",sqlite3_errmsg(db));
	  sqlite3_close(db);
	  db = NULL;
      }
      sqlite3_free(create_sql);
      sqlite3_free_table(sqlresult);
      }
    }

    if (db) {
      /* maximum carid and driverid */
      rc = sqlite3_get_table(db, "SELECT max(id) FROM car", &sqlresult, &sqlrows, &sqlcolumns, NULL);
      if (rc==SQLITE_OK) {
	if ( (sqlcolumns*(sqlrows+1))> 0) {
	  maxid=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
	  if (currentcar > maxid)
	    currentcar=maxid;
	}
	else
	  currentcar=1;
	sqlite3_free_table(sqlresult);
      }
      else
	currentcar=1;

      rc = sqlite3_get_table(db, "SELECT max(id) FROM driver", &sqlresult, &sqlrows, &sqlcolumns, NULL);
      if (rc==SQLITE_OK) {
	if ( (sqlcolumns*(sqlrows+1))> 0) {
	  maxid=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
	  if (currentdriver > maxid)
	    currentdriver=maxid;
	}
	else
	  currentdriver=1;
	sqlite3_free_table(sqlresult);
      }
      else
	currentdriver=1;

      /* prepare sql query for saving the result */
      rc = sqlite3_prepare_v2(db,"INSERT INTO record(carid,driverid,day,km,trip,fill,consum,price,priceperlitre,service,oil,tires,insurance,other,notes) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);",-1,&ppStmtSaveRecord, NULL);

      /* prepare sql query for driver and car selection dialog */
      rc = sqlite3_prepare_v2(db, "SELECT nickname,fullname,id FROM driver;", -1, &ppStmtDriver, NULL);
      rc = sqlite3_prepare_v2(db, "SELECT mark,register,id FROM car;", -1, &ppStmtCar, NULL);

      /* prepare sql query for obtaining one driver data  */
      rc = sqlite3_prepare_v2(db, "SELECT fullname,nickname FROM driver WHERE id=?;", -1, &ppStmtOneDriver, NULL);

      /* prepare sql query for querying the current car */
      rc = sqlite3_prepare_v2(db, "SELECT mark,model,register,year FROM car where id=?;", -1, &ppStmtCurCar, NULL);

      /* prepare sql query for querying the current car */
      rc = sqlite3_prepare_v2(db, "SELECT mark,model,year,register,notes FROM car where id=?;", -1, &ppStmtOneCar, NULL);

      /* prepare sql query for adding a new driver */
      rc = sqlite3_prepare_v2(db, "INSERT INTO driver(fullname,nickname) VALUES(?,?);", -1, &ppStmtAddDriver, NULL);

      /* prepare sql query for changing car properties */
      rc = sqlite3_prepare_v2(db, 
"UPDATE driver "
"SET fullname=?, nickname=? WHERE id=?;",-1, &ppStmtUpdateDriver, NULL);

      /* prepare sql query for adding a new car */
      rc = sqlite3_prepare_v2(db, "INSERT INTO car(mark,model,year,register,notes) VALUES(?,?,?,?,?);", -1, &ppStmtAddCar, NULL);

      /* prepare sql query for changing car properties */
      rc = sqlite3_prepare_v2(db, 
"UPDATE car "
"SET mark=?, model=?, year=?, register=?, notes=? WHERE id=?;",-1, &ppStmtUpdateCar, NULL);

      /* prepare sql query for deleting a record */
      rc = sqlite3_prepare_v2(db, "DELETE FROM record WHERE id=?;", -1, &ppStmtDeleteRecord, NULL);

      /* prepare sql query for obtaining one record */
      rc = sqlite3_prepare_v2(db, 
"SELECT day,km,trip,fill,price,service,oil,tires,notes,id "
"FROM record WHERE id=?;", -1, &ppStmtOneRecord, NULL);

      /* prepare sql query for obtaining all the records */
      rc = sqlite3_prepare_v2(db, 
"SELECT day,km,trip,fill,consum,price,"
"priceperlitre,service,oil,tires,notes,"
"id FROM record WHERE carid=? ORDER BY km;", -1, &ppStmtRecords, NULL);

      /* prepare sql query for obtaining all the records (more fields
         for export functionality) */
      rc = sqlite3_prepare_v2(db, 
"SELECT day,km,trip,fill,consum,price,"
"priceperlitre,service,oil,tires,notes,"
"driverid FROM record WHERE carid=? ORDER BY km;", -1, &ppStmtExport, NULL);

      rc = sqlite3_prepare_v2(db, "SELECT mark,model,year,register,notes FROM car WHERE id=?;", -1, &ppStmtExportCar, NULL);

      /* prepare sql query for adding a record */
      rc = sqlite3_prepare_v2(db, 
"INSERT "
"INTO record(carid,driverid,day,km,trip,fill,consum,price,priceperlitre,service,oil,tires,notes) "
"VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?);", -1, &ppStmtAddRecord, NULL);

      /* prepare sql query for changing a record */
      rc = sqlite3_prepare_v2(db, 
"UPDATE record "
"SET day=?, km=?, trip=?, fill=?, consum=?, price=?, priceperlitre=?, service=?, oil=?, tires=?, notes=? WHERE id=?;", -1, &ppStmtUpdateRecord, NULL);

      /* Prepare sql query for obtaining the years that contain data */
      rc = sqlite3_prepare_v2(db,
"SELECT DISTINCT STRFTIME('%Y',date(day)) FROM record WHERE carid=?;", -1,
			      &ppStmtGetYears, NULL);

      /* Prepare sql query for obtaining the report data */
      rc = sqlite3_prepare_v2(db,
"SELECT STRFTIME('%Y',day),MIN(km),MAX(km),MAX(km)-MIN(km),"
"SUM(fill),SUM(price),SUM(fill)/SUM(trip)*100,SUM(price)/SUM(fill),"
"SUM(price)/SUM(trip),SUM(oil),SUM(oil)/(MAX(km)-MIN(km)),"
"SUM(service),SUM(service)/(MAX(km)-MIN(km)),SUM(tires),"
"SUM(tires)/(MAX(km)-MIN(km)) "
"FROM record WHERE carid=? AND "
/* "STRFTIME('%Y',DATE(day))>= '?' AND " */
/* "STRFTIME('%Y',DATE(day))<= '?' AND " */
"km NOT IN (0) "
"GROUP BY STRFTIME('%Y',DATE(day));", -1, &ppStmtGetReport, NULL);
/* "SELECT STRFTIME('%Y',day),MIN(km),MAX(km),MAX(km)-MIN(km)," */
/* "SUM(fill),SUM(price),SUM(fill)/SUM(trip)*100,SUM(price)/SUM(fill)," */
/* "SUM(price)/SUM(trip),SUM(oil),SUM(oil)/(MAX(km)-MIN(km))," */
/* "SUM(service),SUM(service)/(MAX(km)-MIN(km)),SUM(tires)," */
/* "SUM(tires)/(MAX(km)-MIN(km)) " */
/* "FROM record WHERE carid=? AND STRFTIME('%Y',DATE(day))>='?' AND " */
/* "STRFTIME('%Y',DATE(day))<='?' AND km NOT IN (0) " */
/* "GROUP BY STRFTIME('%Y',DATE(day));", -1, &ppStmtGetReport, NULL); */

      /* prepare sql query for adding an alarmtype */
      rc = sqlite3_prepare_v2(db,
"INSERT "
"INTO alarmtype(carid,shortdesc,distance,interval,longdesc) "
				"VALUES(?,?,?,?,?);", -1, &ppStmtAddAlarmtype, NULL);

      /* prepare sql query for obtaining alarmtypes */
      rc = sqlite3_prepare_v2(db,
"SELECT shortdesc,distance,interval,"
"id FROM alarmtype WHERE carid=?;", -1, &ppStmtGetAlarmtype, NULL);

      /* prepare sql query for obtaining one alarmtype */
      rc = sqlite3_prepare_v2(db,
"SELECT shortdesc,distance,interval,"
"id FROM alarmtype WHERE id=?;", -1, &ppStmtGetOneAlarmtype, NULL);

      /* prepare sql query for updating one alarmtype */
      rc = sqlite3_prepare_v2(db,
"UPDATE alarmtype "
"SET shortdesc=?, distance=?, interval=? WHERE id=?", -1, &ppStmtUpdateAlarmtype, NULL);

      /* prepare sql query for adding an alarm event */
      rc = sqlite3_prepare_v2(db,
"INSERT "
"INTO alarmevent(alarmid,carid,driverid,recordid,day,km) "
			      "VALUES(?,?,?,?,?,?);", -1, &ppStmtAddEvent, NULL);

      /* prepare sql query for changing an alarm event */
      rc = sqlite3_prepare_v2(db, 
"UPDATE alarmevent "
"SET day=?, km=? WHERE id=?;", -1, &ppStmtUpdateEvent, NULL);


      /* prepare sql query for obtaining alarm events */
      rc = sqlite3_prepare_v2(db,
"SELECT day,km,id,recordid FROM alarmevent WHERE alarmid=?;",
			      -1, &ppStmtGetEvents, NULL);

      /* prepare sql query for obtaining one alarm event */
      rc = sqlite3_prepare_v2(db,
"SELECT day,km,recordid FROM alarmevent WHERE id=?;",
			      -1, &ppStmtGetOneEvent, NULL);

      /* prepare sql query for deleting an event */
      rc = sqlite3_prepare_v2(db, "DELETE FROM alarmevent WHERE id=?;", -1, &ppStmtDeleteEvent, NULL);

      /* prepare sql query for deleting an event with a given recordid */
      rc = sqlite3_prepare_v2(db, "DELETE FROM alarmevent WHERE recordid=?;", -1, &ppStmtDeleteEventwithRecordid, NULL);

    }
    /*   sqlite3_close(db); */ /* leave db open! */ 
  }
}  

/**
 * \fn void db_close(void)
 * \brief Closes the connection to sqlite3 database
 *
 */
void db_close(void)
{
  sqlite3_finalize(ppStmtSaveRecord);
  sqlite3_finalize(ppStmtDriver);
  sqlite3_finalize(ppStmtCar);
  sqlite3_finalize(ppStmtOneDriver);
  sqlite3_finalize(ppStmtCurCar);
  sqlite3_finalize(ppStmtOneCar);
  sqlite3_finalize(ppStmtAddDriver);
  sqlite3_finalize(ppStmtUpdateDriver);
  sqlite3_finalize(ppStmtAddCar);
  sqlite3_finalize(ppStmtUpdateCar);
  sqlite3_finalize(ppStmtDeleteRecord);
  sqlite3_finalize(ppStmtOneRecord);
  sqlite3_finalize(ppStmtRecords);
  sqlite3_finalize(ppStmtExport);
  sqlite3_finalize(ppStmtExportCar);
  sqlite3_finalize(ppStmtAddRecord);
  sqlite3_finalize(ppStmtUpdateRecord);
  sqlite3_finalize(ppStmtGetYears);
  sqlite3_finalize(ppStmtGetReport);
  sqlite3_finalize(ppStmtAddAlarmtype);
  sqlite3_finalize(ppStmtGetAlarmtype);
  sqlite3_finalize(ppStmtAddEvent);
  sqlite3_finalize(ppStmtGetEvents);
  sqlite3_finalize(ppStmtGetOneEvent);
  sqlite3_finalize(ppStmtDeleteEvent);
  sqlite3_finalize(ppStmtDeleteEventwithRecordid);
  sqlite3_close(db);
  db = NULL;
}

/**
 * \fn int db_open(void)
 * \brief Tell if the database is open
 * \return True if open, false if not
 */
int db_open(void)
{
  return db != NULL;
}

/**
 * \fn float db_get_last_km(void)
 * \brief Finds the km of last record
 * \return Last km in SI units
 */
float db_get_last_km(void)
{
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
  int sqlcolumns;
  int rc;
  float ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT max(km) FROM record WHERE carid=%d;",currentcar);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
/*    sqlite3_free((void *)sqlquery); */
   sqlite3_free(sqlquery);
   if (rc==SQLITE_OK) {
     if (sqlcolumns*(sqlrows+1)>0) {
       if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
       else
	 ret=0.0;
     }
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0.0;
 }
 else
   ret=0.0;

 return ret;
}

/**
 * \fn float db_get_last_refill(void)
 * \brief Finds the km of last refill
 * \return Last refill km in SI units
 */
float db_get_last_refill(void)
{
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
  int sqlcolumns;
  int rc;
  float ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT km FROM record WHERE carid=%d AND trip>0 ORDER BY km DESC LIMIT 1;",currentcar);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
/*    sqlite3_free((void *)sqlquery); */
   sqlite3_free(sqlquery);
   if (rc==SQLITE_OK) {
     if (sqlcolumns*(sqlrows+1)>0) {
       if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
       else
	 ret=0.0;
     }
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0.0;
 }
 else
   ret=0.0;

 return ret;
}

/**
 * \fn int db_get_monthly_data(int year, double *x, double *y)
 * \brief Get the monthly data of a given year
 * \param year The year the data is requested
 * \param *x   Pointer to month data
 * \param *y   Pointer to consumption data
 * \return Last refill km in SI units
 */
int db_get_monthly_data(int year, double *x, double *y)
{
  int i;
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
  int sqlcolumns;
  int rc;
  int resultsize=0;

  if (db) {
    sqlquery = sqlite3_mprintf("CREATE TEMP VIEW fillview AS SELECT * FROM record WHERE carid=%d ORDER BY km LIMIT -1 OFFSET 1;",currentcar);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
    sqlite3_free((void *)sqlquery);
    sqlite3_free_table(sqlresult);
    if (rc==SQLITE_OK) {
      // select * from fillview where day between '2007-01-01' and '2007-12-31';
      // SELECT STRFTIME(\"%m\",DATE(day)),STRFTIME(\"%Y\",DATE(day)),SUM(fill),SUM(trip),SUM(fill)/SUM(trip)*100 FROM fillview WHERE carid=2 AND STRFTIME(\"%Y\",DATE(day))=='2007' GROUP BY STRFTIME(\"%m\",day);
      sqlquery = sqlite3_mprintf("SELECT STRFTIME(\"%s\",DATE(day)),STRFTIME(\"%s\",DATE(day)),SUM(fill),SUM(trip),SUM(fill)/SUM(trip)*100 FROM fillview WHERE STRFTIME(\"%s\",DATE(day))=='%d' GROUP BY STRFTIME(\"%s\",day);","%m","%Y","%Y",year,"%m");
      rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
      sqlite3_free((void *)sqlquery);
      if (rc==SQLITE_OK) {
	if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) {
	  resultsize=sqlrows;
	  for (i=1;i<=sqlrows;i++) {
	    x[i-1]=atof_l(sqlresult[i*sqlcolumns],"C");
	    y[i-1]=atof_l(sqlresult[i*sqlcolumns+4],"C");
	  }
	}
	else
	  PDEBUG("No data found\n");
	sqlite3_free_table(sqlresult);

	rc = sqlite3_get_table(db, "DROP VIEW fillview;", &sqlresult, &sqlrows, &sqlcolumns, NULL);
	if (rc==SQLITE_OK)
	  sqlite3_free_table(sqlresult);
      } // if(rc==SQLITE_OK)
      else
	PDEBUG("No data found\n");
    } // if(rc==SQLITE_OK)

    return resultsize;
  } // if (db)
  else
    return 0;
}

/**
 * \fn float db_get_totalkm(void)
 * \brief Get the total km of currentcar
 * \return Total km
 */
float db_get_totalkm(void)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 float ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT max(km)-min(km) FROM record WHERE carid=%d;",currentcar);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
/*    sqlite3_free((void *)sqlquery); */
   sqlite3_free(sqlquery);
   if (rc==SQLITE_OK) {
     if (sqlcolumns*(sqlrows+1)>0) {
       if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
       else
	 ret=0.0;
     }
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0.0;
 }
 else
   ret=0.0;

 return ret;
}

/**
 * \fn float db_get_totalcost(void)
 * \brief Get the total cost of currentcar
 * \return Total cost
 */
float db_get_totalcost(void)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 float ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT sum(price)+sum(service)+sum(oil)+sum(tires) FROM record WHERE carid=%d",currentcar);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
/*    sqlite3_free((void *)sqlquery); */
   sqlite3_free(sqlquery);
   if (rc==SQLITE_OK) {
     if (sqlcolumns*(sqlrows+1)>0) {
       if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
       else
	 ret=0.0;
     }
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0.0;
 }
 else
   ret=0.0;

 return ret;
}


/**
 * \fn float db_get_totalfill(void)
 * \brief Get the total fill of currentcar
 * \return Total fill
 *
 * The total fill is calculated as the sum of all fills minus the
 * first fill. This is based on an assumptation that the amount of
 * first fill is still in the tank, so it has not been consumed yet.
 *
 * \todo Wouldn't it be better to subtract the last fill, not the
 * first one?
 *
 */
float db_get_totalfill(void)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 float ret;

 if (db) {
/*    sqlquery = sqlite3_mprintf("CREATE TEMP VIEW fillview AS SELECT fill FROM record WHERE carid=%d ORDER BY km LIMIT -1 OFFSET 1;",currentcar); */
/*    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL); */
/*    sqlite3_free((void *)sqlquery); */
/*    sqlite3_free_table(sqlresult); */
/*    if (rc==SQLITE_OK) { */
/*      printf("succesfully created temporary view\n"); */

/*      rc = sqlite3_get_table(db, "SELECT sum(fill) FROM fillview;", &sqlresult, &sqlrows, &sqlcolumns, NULL); */
/*      printf("sqlrows=%d, sqlcolumns=%d\n",sqlrows,sqlcolumns); */
/*      if (rc==SQLITE_OK) { */
/*        printf("succesfully queried from temporary view\n"); */
/*        if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) */
/* 	 ret=atof(sqlresult[sqlcolumns*(sqlrows+1)-1]); */
/*        else */
/* 	 ret=0.0; */
/*        printf("ret=%f\n",ret); */
/*        sqlite3_free_table(sqlresult); */

/*        rc = sqlite3_get_table(db, "DROP VIEW fillview;", &sqlresult, &sqlrows, &sqlcolumns, NULL); */
/*      if (rc==SQLITE_OK) */
/*        printf("succesfully dropped temporary view\n"); */
/*      sqlite3_free_table(sqlresult); */

/*      } */
/*      else */
/*        ret=0.0; */
/* /\*      sqlite3_free((void *)sqlquery); *\/ */
/*    } */

   sqlquery = sqlite3_mprintf("SELECT sum(fill) FROM record WHERE carid=%d",currentcar);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   sqlite3_free((void *)sqlquery);
   if (rc==SQLITE_OK) {
     if (sqlcolumns*(sqlrows+1)>0) {
       if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) {
	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
	 sqlite3_free_table(sqlresult);
	 sqlquery = sqlite3_mprintf("SELECT fill FROM record WHERE carid=%d ORDER BY km LIMIT 1",currentcar);
	 rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
	 sqlite3_free((void *)sqlquery);
	 if (rc==SQLITE_OK) {
	   if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
	     ret -= atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
	 }
	 sqlite3_free_table(sqlresult);
       }
     }
     else {
       sqlite3_free_table(sqlresult);
       ret=0.0;
     }
   }
   else
     ret=0.0;

 }
 else
   ret=0.0;

 return ret;
}



/**
 * \fn int db_get_last_insert_rowid(void)
 * \brief Get the id of the last insert row
 * \return Rowid of the last inserted row
 *
 * This function uses the sqlite3 function last_insert_rowid() to obtain
 * the id of the last inserted row.
 *
 */
sqlite_int64 db_get_last_insert_rowid(void)
{
 char **sqlresult;
 int sqlrows;
 int sqlcolumns;
 int rc;
 sqlite_int64 ret;

 if (db) {
   rc = sqlite3_get_table(db, "SELECT last_insert_rowid()", &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     ret=atol(sqlresult[sqlcolumns*(sqlrows+1)-1]);
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0;
 }
 else
   ret=0;

 return ret;
}

/**
 * \fn void db_get_driver(char *driver, int id, int n)
 * \brief Get the name of the driver
 * \param *driver pointer the buffer for storing the name
 * \param id the rowid of a record where the driver name is requested
 * \param n max length of the driver buffer
 *
 * This function returns the name of the driver of a given record. The
 * buffer to store the name must have been allocated before calling
 * this function.
 *
 */
void db_get_driver(char *driver, int id, int n)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT nickname FROM driver WHERE id=%d",id);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     if ( (sqlcolumns*(sqlrows+1))> 0)
       strncpy(driver,sqlresult[sqlcolumns*(sqlrows+1)-1],n);
     else
       driver[0]='\0';
     sqlite3_free_table(sqlresult);
   }
   sqlite3_free((void *)sqlquery);
 }
 else
   strcpy(driver,"db closed");
}


/**
 * \fn void db_get_current_driver(char *driver, int n)
 * \brief Get the name of current driver
 * \param *driver pointer the buffer for storing the name
 * \param n max length of the driver buffer
 *
 * This function returns the name of the current driver. The buffer to
 * store the name must have been allocated before calling this
 * function.
 *
 */
void db_get_current_driver(char *driver, int n)
{
  db_get_driver(driver, currentdriver, n);
}

/**
 * \fn int db_get_numof_drivers(void)
 * \brief Get the number of drivers in the database
 * \return The number of drivers in the database
 *
 * This function returns the number of the drivers in the database.
 *
 */
int db_get_numof_drivers(void)
{
 char **sqlresult;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 if (db) {
   rc = sqlite3_get_table(db, "SELECT count(*) FROM driver", &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     ret=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0;
 }
 else
   ret=0;

 return ret;
}

/**
 * \fn int db_get_numof_records(int id)
 * \brief Get the number of records in the database with a given driver
 * \return The number of records in the database with a given driver
 *
 * This function returns the number of records in the database with a
 * given driver.
 *
 */
int db_get_numof_records(int id)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT COUNT(*) FROM record WHERE driverid=%d",id);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     ret=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0;
   sqlite3_free((void *)sqlquery);
 }
 else
   ret=0;

 return ret;
}

/**
 * \fn int db_get_numof_events(int id)
 * \brief Get the number of events in the database with a given driver
 * \return The number of events in the database with a given driver
 *
 * This function returns the number of events in the database with a 
 * given driver.
 *
 */
int db_get_numof_events(int id)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT COUNT(*) FROM alarmevent WHERE driverid=%d",id);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     ret=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0;
   sqlite3_free((void *)sqlquery);
 }
 else
   ret=0;

 return ret;
}

/**
 * \fn db_get_numof_events_with_record(int id)
 * \brief Get the number of events in the database with a corresponding record in the fuel database
 * \return The number of events in the database with a given driver
 *
 * This function returns the number of events in the database for a 
 * corresponding record in the fuel database.
 *
 */
int db_get_numof_events_with_record(int id)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT COUNT(*) FROM alarmevent WHERE recordid=%d",id);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     ret=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0;
   sqlite3_free((void *)sqlquery);
 }
 else
   ret=0;

 return ret;
}

/**
 * \fn int db_get_numof_cars(void)
 * \brief Get the number of cars in the database
 * \return The number of cars in the database
 *
 * This function returns the number of the cars in the database.
 *
 */
int db_get_numof_cars(void)
{
 char **sqlresult;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 if (db) {
   rc = sqlite3_get_table(db, "SELECT count(*) FROM car", &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     ret=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0;
 }
 else
   ret=0;

 return ret;
}

/**
 * \fn int db_get_numof_carrecords(int id)
 * \brief Get the number of records in the database for a given car
 * \return The number of records in the database with a given driver
 *
 * This function returns the number of records in the database with a
 * given driver.
 *
 */
int db_get_numof_carrecords(int id)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT COUNT(*) FROM record WHERE carid=%d",id);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     ret=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0;
   sqlite3_free((void *)sqlquery);
 }
 else
   ret=0;

 return ret;
}

/**
 * \fn int db_get_numof_carevents(int id)
 * \brief Get the number of events in the database with a given car
 * \return The number of events in the database with a given driver
 *
 * This function returns the number of events in the database with a 
 * given driver.
 *
 */
int db_get_numof_carevents(int id)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT COUNT(*) FROM alarmevent WHERE carid=%d",id);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     ret=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0;
   sqlite3_free((void *)sqlquery);
 }
 else
   ret=0;

 return ret;
}

/**
 * \fn int db_get_numof_alarmtypes(int id)
 * \brief Get the number of alarmtypes in the database with a given car
 * \return The number of alarmtypes in the database with a given car
 *
 * This function returns the number of alarmtypes in the database with a 
 * given car.
 *
 */
int db_get_numof_alarmtypes(int id)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT COUNT(*) FROM alarmtype WHERE carid=%d",id);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     ret=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
     sqlite3_free_table(sqlresult);
   }
   else
     ret=0;
   sqlite3_free((void *)sqlquery);
 }
 else
   ret=0;

 return ret;
}

int db_delete_driver(int id)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 ret=0;
 if (db) {
   ret=1;
   sqlquery = sqlite3_mprintf("DELETE FROM driver WHERE id=%d;",id);
   if (SQLITE_OK == sqlite3_exec(db, sqlquery, NULL, NULL, NULL)) {
     sqlite3_free((void *)sqlquery);
     sqlquery = sqlite3_mprintf("DELETE FROM record WHERE driverid=%d;",id);
     if (SQLITE_OK == sqlite3_exec(db, sqlquery, NULL, NULL, NULL)) {
       sqlite3_free((void *)sqlquery);
       sqlquery = sqlite3_mprintf("DELETE FROM alarmevent WHERE driverid=%d;",id);
       if (SQLITE_OK != sqlite3_exec(db, sqlquery, NULL, NULL, NULL)) 
	 ret=0;
     }
     else
       ret=0;
   }
   else
     ret=0;
 }
 return ret;
}

int db_delete_car(int id)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int ret;

 ret=0;
 if (db) {
   ret=1;
   sqlquery = sqlite3_mprintf("DELETE FROM car WHERE id=%d;",id);
   if (SQLITE_OK == sqlite3_exec(db, sqlquery, NULL, NULL, NULL)) {
     sqlite3_free((void *)sqlquery);
     sqlquery = sqlite3_mprintf("DELETE FROM record WHERE carid=%d;",id);
     if (SQLITE_OK == sqlite3_exec(db, sqlquery, NULL, NULL, NULL)) {
       sqlite3_free((void *)sqlquery);
       sqlquery = sqlite3_mprintf("DELETE FROM alarmevent WHERE carid=%d;",id);
       if (SQLITE_OK != sqlite3_exec(db, sqlquery, NULL, NULL, NULL)) 
	 ret=0;
     }
     else
       ret=0;
   }
   else
     ret=0;
 }
 return ret;
}

/**
 * \fn void db_get_current_car(char *car, int n)
 * \brief Get the name of current car
 * \param *car pointer the buffer for storing the name
 * \param n max length of the car buffer
 *
 * This function returns the name of the current car. The buffer to
 * store the name must have been allocated before calling this
 * function.
 *
 */
void db_get_current_car(char *car, int n)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT mark FROM car WHERE id=%d",currentcar);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     if ( (sqlcolumns*(sqlrows+1))> 0)
       strncpy(car,sqlresult[sqlcolumns*(sqlrows+1)-1],n);
     else
       car[0]='\0';
     sqlite3_free_table(sqlresult);
   }
   sqlite3_free((void *)sqlquery);
 }
 else
   strcpy(car,"db closed");
}

/**
 * \fn int db_estimate_date(float km, char *estdate, int n, char *lastdate, double interval)
 * \brief Estimate the date when the given km will be reached
 * \param km the given km
 * \param *estdate pointer the buffer for storing the date
 * \param n max length of the date buffer
 * \param *lastdate date of last event
 * \param interval interval between services in months
 *
 * This function estimates when the given km will be reached. The
 * buffer to store the date must have been allocated before calling
 * this function.
 *
 */
int db_estimate_date(float km, char *estdate, int n, char *lastdate, double interval)
{
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
  int sqlcolumns;
  int rc;
  int ret;

  ret = 1;
  if (db) {
    sqlquery = sqlite3_mprintf("CREATE TEMP VIEW kmview AS SELECT * FROM record WHERE carid=%d ORDER BY km DESC LIMIT 5;",currentcar);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
    sqlite3_free((void *)sqlquery);
    sqlite3_free_table(sqlresult);
    if (rc==SQLITE_OK) {
      if (lastdate == NULL || strlen(lastdate)==0 || fabs(interval)<0.01)
	sqlquery = sqlite3_mprintf("SELECT DATE(DATE(MAX(day)), QUOTE((%f-MAX(km))/((MAX(km)-MIN(km))/((STRFTIME(\"%s\",DATE(MAX(day)))-STRFTIME(\"%s\",DATE(MIN(day))))/3600)))||'hours') FROM kmview;",km,"%s","%s");
      else
	sqlquery = sqlite3_mprintf("SELECT MIN(DATE(\"%s\",\"%f\"||\"months\"),DATE(DATE(MAX(day)), QUOTE((%f-MAX(km))/((MAX(km)-MIN(km))/((STRFTIME(\"%s\",DATE(MAX(day)))-STRFTIME(\"%s\",DATE(MIN(day))))/3600)))||'hours')) FROM kmview;",lastdate,interval,km,"%s","%s");
      rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
      sqlite3_free((void *)sqlquery);
      if (rc==SQLITE_OK) {
	if ( (sqlcolumns*(sqlrows+1))> 0) {
	  if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
	    strncpy(estdate,sqlresult[sqlcolumns*(sqlrows+1)-1],n);
	  else
	    estdate[0]='\0';
	}
	else {
	  PDEBUG("No data found\n");
	  estdate[0]='\0';
	  ret=0;
	}
	sqlite3_free_table(sqlresult);
      }

      rc = sqlite3_get_table(db, "DROP VIEW kmview;", &sqlresult, &sqlrows, &sqlcolumns, NULL);
      if (rc==SQLITE_OK) {
	sqlite3_free_table(sqlresult);
	ret=0;
      }
    }
    else {
      estdate[0]='\0';
      ret=0;
    }
  }

  return ret;
}

/**
 * \fn float db_estimate_km(char *lastdate, double interval)
 * \brief Estimate the date when the given km will be reached
 * \param *lastdate date of last event
 * \param interval interval between services in months
 *
 * This function estimates the km at time estdate + interval months.
 *
 */
float db_estimate_km(char *lastdate, double interval)
{
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
  int sqlcolumns;
  int rc;
  float ret;

  ret = 0.0;
  if (db) {
    sqlquery = sqlite3_mprintf("CREATE TEMP VIEW kmview AS SELECT * FROM record WHERE carid=%d ORDER BY km DESC LIMIT 5;",currentcar);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
    sqlite3_free((void *)sqlquery);
    sqlite3_free_table(sqlresult);
    if (rc==SQLITE_OK) {
      if (lastdate != NULL) {
	sqlquery = sqlite3_mprintf("SELECT MIN(km)+(MAX(km)-MIN(km))/(STRFTIME(\"%s\",DATE(MAX(day)))-STRFTIME(\"%s\",DATE(MIN(day))))*(STRFTIME(\"%s\",DATE(\"%s\",'+%d months'))-STRFTIME(\"%s\",DATE(MIN(day)))) FROM kmview;","%s","%s","%s",lastdate,(int)interval,"%s");
	rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
	sqlite3_free((void *)sqlquery);
	if (rc==SQLITE_OK) {
	  ret=atof(sqlresult[sqlcolumns*(sqlrows+1)-1]);
	  sqlite3_free_table(sqlresult);
	}
      }
    }

    rc = sqlite3_get_table(db, "DROP VIEW kmview;", &sqlresult, &sqlrows, &sqlcolumns, NULL);
    if (rc==SQLITE_OK)
      sqlite3_free_table(sqlresult);
  }

  return ret;
}

void db_get_last_event(int alarmid, char *day, double *km, int n)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT day, km FROM alarmevent WHERE alarmid=%d ORDER BY km DESC LIMIT 1",alarmid);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     if ( (sqlcolumns*(sqlrows+1))> 0) {
       strncpy(day,sqlresult[sqlcolumns*(sqlrows+1)-2],n);
       *km=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
     }
     else {
       day[0]='\0';
       *km=0.0;
     }
     sqlite3_free_table(sqlresult);
   }
   sqlite3_free((void *)sqlquery);
 }
 else {
   strcpy(day,"db closed");
   *km=0.0;
 }
}

void db_calc_date(char *newdate, char *date, double num, char *unit, int n)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT DATE(\"%s\",'%d'||'%s');",date,(int)num,unit);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     if ( (sqlcolumns*(sqlrows+1))> 0) {
       strncpy(newdate,sqlresult[sqlcolumns*(sqlrows+1)-1],n);
     }
     else {
       newdate[0]='\0';
     }
     sqlite3_free_table(sqlresult);
   }
   sqlite3_free((void *)sqlquery);
 }
 else {
   strcpy(newdate,"db closed");
 }
}

int db_events_exist(int alarmid)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int retval;

 if (db) {
   sqlquery = sqlite3_mprintf("SELECT id FROM alarmevent WHERE alarmid=%d LIMIT 1",alarmid);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     if ( (sqlcolumns*(sqlrows+1))> 0) {
       retval=1;
     }
     else {
       retval=0;
     }
     sqlite3_free_table(sqlresult);
   }
   sqlite3_free((void *)sqlquery);
 }
 else {
   retval=0;
 }
 return retval;
}

int db_delete_events(int alarmid)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int retval;

 if (db) {
   sqlquery = sqlite3_mprintf("DELETE FROM alarmevent WHERE alarmid=%d",alarmid);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     if ( (sqlcolumns*(sqlrows+1))> 0) {
       retval=1;
     }
     else {
       retval=0;
     }
     sqlite3_free_table(sqlresult);
   }
   sqlite3_free((void *)sqlquery);
 }
 else {
   retval=0;
 }
 return retval;
}

int db_delete_reminder(int id)
{
 char **sqlresult;
 char *sqlquery;
 int sqlrows;
 int sqlcolumns;
 int rc;
 int retval;

 if (db) {
   sqlquery = sqlite3_mprintf("DELETE FROM alarmtype WHERE id=%d",id);
   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
   if (rc==SQLITE_OK) {
     if ( (sqlcolumns*(sqlrows+1))> 0) {
       retval=1;
     }
     else {
       retval=0;
     }
     sqlite3_free_table(sqlresult);
   }
   sqlite3_free((void *)sqlquery);
 }
 else {
   retval=0;
 }
 return retval;
}


/**
 * \fn void fuelpad_exit(void)
 * \brief Does the exit of application cleanly
 *
 * Closes the connection to sqlite3 database.
 *
 */
void fuelpad_exit(void)
{
  db_close();
}

int main( int   argc,
          char *argv[] )
{
  int c;

  while ((c=getopt(argc, argv, "D"))>-1) {
    switch (c) {
    case 'D':
      debug_level = 1;
      break;
    default:
      printf("Invalid option found.\n");
      break;
    }
  }

  db_setfilename(default_db);

  ui_init(argc,argv);

  db_connect();

  ui_create_mainwin();

  ui_main_loop();

  fuelpad_exit();

  return 0;
}
